﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;

namespace HeuristicLab.Visualization {
  public class ZoomInChartMode : ChartMode {
    public override Image Image { get { return VSImageLibrary.ZoomIn; } }
    public override string Text { get { return "Zoom &In"; } }
    public override string ToolTipText { get { return "Zoom In"; } }
    public override Cursor Cursor {
      get { return cursor ?? (cursor = new Cursor(typeof(ChartControl), "Resources.ZoomIn.cur")); }
    }

    public ZoomInChartMode(ChartControl control) : base(control) { }

    public override void HandleOnMouseUp(object sender, MouseEventArgs e) {
      try {
        switch (e.Button) {
          case MouseButtons.Left:
            var lowerLeft = new Point(Math.Min(e.X, buttonDownPoint.X), Math.Max(e.Y, buttonDownPoint.Y));
            var upperRight = new Point(Math.Max(e.X, buttonDownPoint.X), Math.Min(e.Y, buttonDownPoint.Y));
            if ((lowerLeft.X != upperRight.X) && (lowerLeft.Y != upperRight.Y))
              chartControl.Chart.ZoomIn(lowerLeft, upperRight);
            else chartControl.Chart.ZoomIn(buttonDownPoint);
            break;
          case MouseButtons.Middle:
            chartControl.Chart.Unzoom();
            break;
        }
      } finally {
        base.HandleOnMouseUp(sender, e);
      }
    }

    public override void HandleOnMouseMove(object sender, MouseEventArgs e) {
      try {
        switch (e.Button) {
          case MouseButtons.Left:
            chartControl.RefreshPicture();
            using (var graphics = chartControl.CreatePictureGraphics())
            using (var pen = new Pen(Color.Gray) { DashStyle = DashStyle.Dash }) {
              graphics.DrawRectangle(pen,
                Math.Min(e.X, buttonDownPoint.X),
                Math.Min(e.Y, buttonDownPoint.Y),
                Math.Abs(e.X - buttonDownPoint.X),
                Math.Abs(e.Y - buttonDownPoint.Y));
            }
            break;
        }
      } finally {
        chartControl.UpdatePicture();
        base.HandleOnMouseMove(sender, e);
      }
    }
  }
}
